using System;
using System.Collections.Generic;
using System.ServiceProcess;
using System.Text;
using System.Net.Sockets;
using System.Net;
using System.Threading;
using System.Collections;
using Alchemi.Core;
using Alchemi.Core.Owner;
using Alchemi.Core.Utility;
using log4net;
using System.Data.SqlClient;
using System.Data;

namespace TaskExecutor
{
    public partial class TaskExecutor : ServiceBase
    {
        #region Alchemi variables

        GApplication App;
        DateTime StartTime;
        BigInteger max = new BigInteger("0", 10);
        ArrayList results = new ArrayList();
        ArrayList results2 = new ArrayList();
        ArrayList factors = new ArrayList();
        bool started = false;
        ulong threadcount;
        StringBuilder progress = new StringBuilder("Aplikacia");

        #endregion

        Socket listener;
        Thread listenerThread;
        Thread senderThread;

        public TaskExecutor()
        {
            InitializeComponent();
        }

        protected override void OnStart(string[] args)
        {
            progress.Remove(0, progress.Length);
            progress.Append("Aplikcia bola inicializovan");
            listenerThread = new Thread(new ThreadStart(this.Listen));
            listenerThread.Start();
            senderThread = new Thread(new ThreadStart(this.Send));
            senderThread.Start();
        }

        protected override void OnStop()
        {
            try
            {
                App.Stop();
                progress.AppendLine();
                progress.AppendLine("Vpoet bol ukonen");
                listenerThread.Abort();
                listener.Close();
                senderThread.Abort();
                progress.Remove(0, progress.Length);
            }
            catch
            {

            }
        }

        protected void Listen()
        {
            string message = "";
            try
            {
                listener = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                listener.Bind(new System.Net.IPEndPoint(System.Net.IPAddress.Any, 8000));
                listener.Listen(0);
                byte[] buffer;
                UnicodeEncoding encoder;

                while (true)
                {
                    Socket socket = listener.Accept();
                    byte[] flag = new byte[1024];
                    int length = socket.Receive(flag);
                    switch (flag[0])
                    {
                        case 0:
                            message = "Aplikacia bola uspesne spustena";
                            progress.Remove(0, progress.Length);
                            string[] temp = Encoding.ASCII.GetString(flag, 1, length - 1).Split(' ');
                            StartGrid(temp[0], temp[1]);
                            started = true;
                            break;
                        case 2:
                            message = "Aplikacia bola uspesne ukoncena";
                            App.Stop();
                            progress.Remove(0, progress.Length);
                            started = false;
                            break;
                    }
                    encoder = new UnicodeEncoding();
                    buffer = encoder.GetBytes(message);
                    socket.Send(buffer, buffer.Length, 0);
                    socket.Close();
                }
            }
            finally
            {
                listenerThread.Abort();
                listener.Close();
            }
        }

        private void Send()
        {
            UdpClient client = new UdpClient();
            try
            {
                while (true)
                {
                    byte[] progress1 = Encoding.Default.GetBytes(progress.ToString());
                    byte[] started1 = Encoding.Default.GetBytes(started.ToString());
                    byte[] separator = Encoding.Default.GetBytes("@");
                    byte[] number = Encoding.Default.GetBytes(max.ToString());
                    byte[] message = new byte[progress.Length + separator.Length + started1.Length + separator.Length + number.Length];
                    progress1.CopyTo(message, 0);
                    separator.CopyTo(message, progress1.Length);
                    started1.CopyTo(message, progress1.Length + separator.Length);
                    separator.CopyTo(message, progress1.Length + separator.Length + started1.Length);
                    number.CopyTo(message, progress1.Length + separator.Length + started1.Length + separator.Length);
                    //client.Send(progress, progress.Length, "localhost", 8000);
                    client.Send(message, message.Length, new IPEndPoint(IPAddress.Parse("127.0.0.1"), 8000));
                    Thread.Sleep(1000);
                }
            }
            finally
            {
                senderThread.Abort();
                client.Close();
            }
        }

        #region Alchemi

        public void StartGrid(string inputMax, string threadcountString)
        {
            try
            {
                max = new BigInteger(inputMax, 10);
                threadcount = Convert.ToUInt64(threadcountString);
            }
            catch { }

            if (threadcount == 0) threadcount = 10;

            started = true;
            Initialize();
            BigInteger number = max.sqrt();
            Random rnd = new Random();
            BigInteger intervalLength = number / threadcount;
            for (BigInteger i = new BigInteger(); i <= number; i = i + intervalLength)
            {
                BigInteger intStart = i;
                BigInteger intEnd = i + intervalLength;
                if (intEnd > number) intEnd = number;

                progress.AppendFormat("Creating a grid thread to check the interval {0} - {1}", intStart, intEnd);
                progress.AppendLine();
                App.Threads.Add(new FactorChecker(max, intStart, intEnd));
                i++;
            }

            App.Start();
        }

        private void Initialize()
        {
            try
            {
                GConnection gc = new GConnection("localhost", 9000, "user", "user");
                StartTime = DateTime.Now;
                App = new GApplication(gc);

                App.ApplicationName = "Number Factorization";
                App.Manifest.Add(new ModuleDependency(typeof(FactorChecker).Module));

                App.ThreadFinish += new GThreadFinish(ThreadFinished);
                App.ApplicationFinish += new GApplicationFinish(ApplicationFinished);
            }
            catch (Exception ex)
            {
                progress.AppendLine("Error: " + ex.Message);
            }
        }

        private void ApplicationFinished()
        {
            if (factors.Count != 0)
            {
                for (int i = factors.Count - 1; i >= 0; i--)
                {
                    for (int j = i - 1; j >= 0; j--)
                    {
                        if ((BigInteger)factors[j] == (BigInteger)factors[i])
                            continue;
                        if ((BigInteger)factors[i] % (BigInteger)factors[j] == 0)
                        {
                            factors.Remove(factors[i]);
                            i = factors.Count;
                            break;
                        }
                    }
                }
            }

            string sqlConnection = @"
            Delete from application where application_id = '" + App.Id + @"'
            Delete from thread where application_id = '" + App.Id + "'";

            SqlConnection connection = new SqlConnection(DBConn.connStringAlchemi);
            try
            {
                connection.Open();
                SqlCommand cmd = new SqlCommand(sqlConnection, connection);
                cmd.CommandType = CommandType.Text;
                cmd.ExecuteNonQuery();
            }
            finally { connection.Close(); }

            progress.AppendLine();
            progress.AppendLine("These are the results - factors:");
            for (int i = 0; i < factors.Count; i++)
            {
                progress.AppendFormat("{0}, ", (BigInteger)factors[i]);
            }
            progress.AppendLine();

            progress.AppendFormat("Factors found: some (kind of). Total time taken : {0}", DateTime.Now - StartTime);
            progress.AppendLine();
            progress.AppendLine("Application Finished!");
            factors.Clear();
        }

        private void CheckResults(int probe)
        {
            int[] resultsTemp = (int[])results2.ToArray(typeof(int));
            for (int i = 0; i <= resultsTemp.Length; i++)
            {
                if (resultsTemp[i] % probe != 0)
                {
                    results2.Add(probe);
                }
            }
        }

        private void ThreadFinished(GThread thread)
        {
            FactorChecker pnc = (FactorChecker)thread;

            progress.AppendFormat("Z intervalu {0}-{1} je faktorizacia: ", pnc.intStart, pnc.intEnd);
            foreach (object o in pnc.factors)
            {
                if ((BigInteger)o != 1)
                    factors.Add(o);
            }

            for (int i = 0; i < pnc.factors.Count; i++)
            {
                if ((BigInteger)pnc.factors[i] == 1) continue;
                progress.AppendFormat("{0}, ", pnc.factors[i]);
            }
            progress.AppendLine("");
            Console.Clear();
            Console.WriteLine(progress);
        }

        #endregion
    }
}
